/*
 * Copyright © 2022 Mark Raynsford <code@io7m.com> https://www.io7m.com
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */


package com.io7m.exfilac.service.api;

/**
 * The type of (writable) service directories.
 */

public interface RPServiceDirectoryWritableType extends RPServiceDirectoryType {
  /**
   * Register the given service.
   *
   * @param clazz   The service interface
   * @param service The service
   * @param <T>     The type of service
   */

  <T extends RPServiceType> void register(
    Class<T> clazz,
    T service);

  /**
   * Deregister the given service.
   *
   * @param clazz   The service interface
   * @param service The service
   * @param <T>     The type of service
   */

  <T extends RPServiceType> void deregister(
    Class<T> clazz,
    T service);

  /**
   * Deregister all service instance of the given type.
   *
   * @param clazz The service interface
   * @param <T>   The type of service
   */

  <T extends RPServiceType> void deregisterAll(
    Class<T> clazz);
}
